@@ -6,7 +6,7 @@ from . import utils
 from .msg import err_buf
 
 
-cib_verify = "crm_verify --verbose -p"
+cib_verify = "crm_verify -VV -p"
 VALIDATE_RE = re.compile(r"^Entity: line (\d)+: element (\w+): " +
                          r"Relax-NG validity error : (.+)$")
 
@@ -19,7 +19,7 @@ def _prettify(line, indent=0):
 
 
 def verify(cib):
-    rc, _, stderr = utils.get_stdout_stderr(cib_verify, cib)
+    rc, _, stderr = utils.get_stdout_stderr(cib_verify, cib.encode('utf-8'))
     for i, line in enumerate(line for line in stderr.split('\n') if line):
         if i == 0:
             err_buf.error(_prettify(line, 0))
