@@ -7,6 +7,7 @@
 #   inside the functions.
 
 import inspect
+import re
 from . import help as help_module
 from . import ui_utils
 from .msg import common_debug
@@ -98,7 +99,7 @@ def skill_level(new_level):
         def do_rmrf(self, cmd, args):
             ...
     '''
-    if isinstance(new_level, basestring):
+    if isinstance(new_level, str):
         levels = {'operator': 0, 'administrator': 1, 'expert': 2}
         if new_level.lower() not in levels:
             raise ValueError("Unknown skill level: " + new_level)
@@ -143,7 +144,7 @@ def completers(*fns):
     Each completer gets as its argument the command line entered so far,
     and returns a list of possible completions.
     '''
-    def completer(args):
+    def cfn(args):
         nargs = len(args) - 1
         if nargs == 0:
             return [args[0]]
@@ -152,7 +153,7 @@ def completers(*fns):
         return []
 
     def inner(fn):
-        setattr(fn, '_completer', completer)
+        setattr(fn, '_completer', cfn)
         return fn
     return inner
 
@@ -162,7 +163,7 @@ def completers_repeating(*fns):
     Like completers, but calls the last completer
     for any additional arguments
     '''
-    def completer(args):
+    def cfn(args):
         nargs = len(args) - 1
         if nargs == 0:
             return [args[0]]
@@ -171,18 +172,56 @@ def completers_repeating(*fns):
         return fns[-1](args)
 
     def inner(fn):
-        setattr(fn, '_completer', completer)
+        setattr(fn, '_completer', cfn)
         return fn
     return inner
 
 
 def _cd_completer(args, context):
-    'TODO: make better completion'
+    """
+    more like bash cd completion
+    """
+    def current_completions():
+        return context.current_level().get_completions()
+
+    def is_sublevel(l):
+        return context.current_level().is_sublevel(l)
+
+    def next_completions(token):
+        info = context.current_level().get_child(token)
+        context.enter_level(info.level)
+        return [l for l in current_completions() if is_sublevel(l)]
+
+    def prev_completions():
+        return [l for l in context.previous_level().get_completions()
+                if context.previous_level().is_sublevel(l)]
+
+    if len(args) == 1 and args[0] == 'cd':
+        # complete the 'cd' command self
+        return current_completions()
+    if len(args) == 2:
+        if args[1] in current_completions():
+            return [args[1] + '/']
+        if args[1] == '..' and context.previous_level():
+            return ['../']
+        if args[1] == '../' and context.previous_level():
+            return [args[1] + l for l in prev_completions()]
+        if args[1].endswith("/"):
+            return [args[1]+l for l in next_completions(args[1].strip('/'))]
+        if re.search(r'\.\./.+', args[1]):
+            return ['../' + l for l in prev_completions()]
+        if re.search(r'.+/.+', args[1]):
+            token = args[1].split('/')[0]
+            return [token+'/'+l for l in next_completions(token)]
+    if len(args) == 3 and not args[-1]:
+        # prevent '..' completion happend many times triggerd by Tab
+        return
+
     ret = []
     if context.previous_level():
         ret += ['..']
-    return ret + [l for l in context.current_level().get_completions()
-                  if context.current_level().is_sublevel(l)]
+    # look out where 'cd' command can enter
+    return ret + [l for l in current_completions() if is_sublevel(l)]
 
 
 def _help_completer(args, context):
@@ -202,13 +241,12 @@ def fuzzy_get(items, s):
     found = items.get(s)
     if found:
         return found
-    import re
 
     def fuzzy_match(rx):
         try:
             matcher = re.compile(rx, re.I)
             matches = [c
-                       for m, c in items.iteritems()
+                       for m, c in items.items()
                        if matcher.match(m)]
             if len(matches) == 1:
                 return matches[0]
@@ -283,18 +321,28 @@ at the current level.
         Shows list of places to go and commands to call
         '''
         out = []
+        res = []
+        max_width = 16
         if context.previous_level():
             out = ['..']
         out += context.current_level().get_completions()
-        i = 0
-        for o in out:
+        for o in sorted(out):
             if o.startswith('-') or o.startswith('_'):
                 continue
-            print '%-16s' % (o),
-            if ((i - 2) % 3) == 0:
-                print ''
-            i += 1
-        print ''
+            if max_width < len(o):
+                max_width = len(o)
+            res.append(o)
+
+        if max_width >= 16:
+            max_width += 2
+
+        colnum = 3
+        rownum = len(res) // colnum
+        for i in range(rownum):
+            for x in res[i::rownum]:
+                print("%-0*s" % (max_width, x), end=' ')
+            print('')
+
 
     @help('''Navigate the level structure
 This command works similar to how `cd` works in a regular unix
@@ -366,12 +414,13 @@ Examples:
         """usage: help topic|level|command"""
         h = help_module.help_contextual(context.level_name(), subject, subtopic)
         h.paginate()
+        context.command_name = ""
 
     def get_completions(self):
         '''
         return tab completions
         '''
-        return self._children.keys()
+        return [x for x in self._children.keys() if x not in self._aliases]
 
     def get_child(self, child):
         '''
@@ -404,11 +453,12 @@ Examples:
         def get_if_command(attr):
             "Return the named attribute if it's a command"
             child = getattr(cls, attr)
-            return child if attr.startswith('do_') and inspect.ismethod(child) else None
+            return child if attr.startswith('do_') and inspect.isfunction(child) else None
 
-        def add_aliases(children, info):
+        def add_aliases(children, info, aliases):
             "Add any aliases for command to child map"
             for alias in info.aliases:
+                aliases.append(alias)
                 children[alias] = info
 
         def add_help(info):
@@ -431,21 +481,25 @@ Examples:
             elif info.type == 'level':
                 help_module.add_help(entry, level=info.name)
 
-        def prepare(children, child):
+        def prepare(children, child, aliases):
             info = ChildInfo(child, cls)
             if info.type == 'command' and not is_valid_command_function(info.function):
                 raise ValueError("Invalid command function: %s.%s" %
                                  (cls.__name__, info.function.__name__))
             children[info.name] = info
-            add_aliases(children, info)
+            add_aliases(children, info, aliases)
             add_help(info)
 
         children = {}
+        aliases = []
         for child_name in dir(cls):
+            if child_name == 'do_up' and re.search("ui_root.Root", str(cls)):
+                continue
             child = get_if_command(child_name)
             if child:
-                prepare(children, child)
+                prepare(children, child, aliases)
         setattr(cls, '_children', children)
+        setattr(cls, '_aliases', aliases)
         return children
 
 
@@ -509,7 +563,7 @@ class ChildInfo(object):
         '''
         ret = []
         if self.completer is not None:
-            specs = inspect.getargspec(self.completer)
+            specs = inspect.getfullargspec(self.completer)
             if 'context' in specs.args:
                 ret = self.completer([self.name] + args, context)
             else:
@@ -525,12 +579,12 @@ def is_valid_command_function(fn):
     Returns True if fn is a valid command function:
     named do_xxx, takes (self, context) as the first two parameters
     '''
-    specs = inspect.getargspec(fn)
+    specs = inspect.getfullargspec(fn)
     return len(specs.args) >= 2 and specs.args[0] == 'self' and specs.args[1] == 'context'
 
 
 def _check_args(fn, expected):
-    argnames = fn.func_code.co_varnames[:fn.func_code.co_argcount]
+    argnames = fn.__code__.co_varnames[:fn.__code__.co_argcount]
     if argnames != expected:
         raise ValueError(fn.__name__ +
                          ": Expected method with signature " + repr(expected))
