@@ -16,7 +16,7 @@ def choice(lst):
 
 
 null = choice([])
-
+attr_id = choice(["id="])
 
 def call(fn, *fnargs):
     '''
@@ -44,12 +44,28 @@ def join(*fns):
 booleans = choice(['yes', 'no', 'true', 'false', 'on', 'off'])
 
 
-def resources(args):
+def resources(args=None):
     cib_el = xmlutil.resources_xml()
     if cib_el is None:
         return []
     nodes = xmlutil.get_interesting_nodes(cib_el, [])
-    return [x.get("id") for x in nodes if xmlutil.is_resource(x)]
+    res = [x.get("id") for x in nodes if xmlutil.is_resource(x)]
+    if args and args[0] in ['promote', 'demote']:
+        return list(filter(xmlutil.RscState().is_ms, res))
+    if args and args[0] == "started":
+        return list(filter(xmlutil.RscState().is_running, res))
+    if args and args[0] == "stopped":
+        for item in filter(xmlutil.RscState().is_running, res):
+            res.remove(item)
+    return res
+
+
+def resources_started(args):
+    return resources(["started"])
+
+
+def resources_stopped(args):
+    return resources(["stopped"])
 
 
 def primitives(args):
@@ -59,6 +75,11 @@ def primitives(args):
     nodes = xmlutil.get_interesting_nodes(cib_el, [])
     return [x.get("id") for x in nodes if xmlutil.is_primitive(x)]
 
+
 nodes = call(xmlutil.listnodes)
 
 shadows = call(xmlutil.listshadows)
+
+status_option = """full bynode inactive ops timing failcounts
+                   verbose quiet xml simple tickets noheaders
+                   detail brief""".split()
