@@ -6,10 +6,7 @@ Holds user-configurable options.
 
 import os
 import re
-try:
-    import ConfigParser
-except ImportError:
-    import configparser as ConfigParser
+import configparser
 from . import userdir
 
 
@@ -188,7 +185,7 @@ class opt_dir(object):
 
     def validate(self, val):
         if not os.path.isdir(val):
-            raise ValueError("Directory not found: " % (val))
+            raise ValueError("Directory not found: %s" % (val))
 
     def get(self, value):
         return value
@@ -275,6 +272,14 @@ DEFAULTS = {
         'id_reference': opt_color('green'),
         'score': opt_color('magenta'),
         'ticket': opt_color('magenta'),
+    },
+    'report': {
+        'from_time': opt_string('-12H'),
+        'compress': opt_boolean('yes'),
+        'speed_up': opt_boolean('no'),
+        'collect_extra_logs': opt_string('/var/log/messages /var/log/pacemaker/pacemaker.log /var/log/pacemaker.log /var/log/ha-cluster-bootstrap.log'),
+        'remove_exist_dest': opt_boolean('no'),
+        'single_node': opt_boolean('no')
     }
 }
 
@@ -286,7 +291,7 @@ def _stringify(val):
         return 'true'
     elif val is False:
         return 'false'
-    elif isinstance(val, basestring):
+    elif isinstance(val, str):
         return val
     else:
         return str(val)
@@ -299,21 +304,21 @@ class _Configuration(object):
         self._user = None
 
     def load(self):
-        self._defaults = ConfigParser.SafeConfigParser()
-        for section, keys in DEFAULTS.iteritems():
+        self._defaults = configparser.SafeConfigParser()
+        for section, keys in DEFAULTS.items():
             self._defaults.add_section(section)
-            for key, opt in keys.iteritems():
+            for key, opt in keys.items():
                 self._defaults.set(section, key, opt.default)
 
         if os.path.isfile(_SYSTEMWIDE):
-            self._systemwide = ConfigParser.SafeConfigParser()
+            self._systemwide = configparser.SafeConfigParser()
             self._systemwide.read([_SYSTEMWIDE])
         # for backwards compatibility with <=2.1.1 due to ridiculous bug
         elif os.path.isfile("/etc/crm/crmsh.conf"):
-            self._systemwide = ConfigParser.SafeConfigParser()
+            self._systemwide = configparser.SafeConfigParser()
             self._systemwide.read(["/etc/crm/crmsh.conf"])
         if os.path.isfile(_PERUSER):
-            self._user = ConfigParser.SafeConfigParser()
+            self._user = configparser.SafeConfigParser()
             self._user.read([_PERUSER])
 
     def save(self):
@@ -331,7 +336,7 @@ class _Configuration(object):
             if self._systemwide and self._systemwide.has_option(section, name):
                 return self._systemwide.get(section, name) or ''
             return self._defaults.get(section, name) or ''
-        except ConfigParser.NoOptionError as e:
+        except configparser.NoOptionError as e:
             raise ValueError(e)
 
     def get(self, section, name, raw=False):
@@ -346,7 +351,7 @@ class _Configuration(object):
             raise ValueError("Setting invalid option %s.%s" % (section, name))
         DEFAULTS[section][name].validate(value)
         if self._user is None:
-            self._user = ConfigParser.SafeConfigParser()
+            self._user = configparser.SafeConfigParser()
         if not self._user.has_section(section):
             self._user.add_section(section)
         self._user.set(section, name, _stringify(value))
@@ -364,7 +369,7 @@ class _Configuration(object):
 
     def reset(self):
         '''reset to what is on disk'''
-        self._user = ConfigParser.SafeConfigParser()
+        self._user = configparser.SafeConfigParser()
         self._user.read([_PERUSER])
 
 
@@ -414,8 +419,8 @@ def get_option(section, option, raw=False):
 def get_all_options():
     '''Returns a list of all configurable options'''
     ret = []
-    for sname, section in DEFAULTS.iteritems():
-        ret += ['%s.%s' % (sname, option) for option in section.keys()]
+    for sname, section in DEFAULTS.items():
+        ret += ['%s.%s' % (sname, option) for option in list(section.keys())]
     return sorted(ret)
 
 
@@ -450,18 +455,17 @@ load()
 core = _Section('core')
 path = _Section('path')
 color = _Section('color')
+report = _Section('report')
 
 
 def load_version():
     version = 'dev'
     versioninfo_file = os.path.join(path.sharedir, 'version')
     if os.path.isfile(versioninfo_file):
-        v = open(versioninfo_file).xreadlines()
-        try:
-            version = v.next().strip() or version
-        except StopIteration:
-            pass
+        with open(versioninfo_file) as f:
+            version = f.readline().strip() or version
     return version
 
+
 VERSION = load_version()
 CRM_VERSION = str(VERSION)
