@@ -219,7 +219,7 @@ def render_prompt(context):
     if options.interactive and not options.batch:
         # TODO: fix how color interacts with readline,
         # seems the color prompt messes it up
-        promptstr = "crm(%s)%s# " % (cib_prompt(), context.prompt())
+        promptstr = "crm(%s/%s)%s# " % (cib_prompt(), utils.this_node(), context.prompt())
         constants.prompt = promptstr
         if clidisplay.colors_enabled():
             rendered_prompt = term.render(clidisplay.prompt(promptstr))
@@ -268,7 +268,7 @@ def main_input_loop(context, user_args):
                 common_err(msg)
         except KeyboardInterrupt:
             if options.interactive and not options.batch:
-                print "Ctrl-C, leaving"
+                print("Ctrl-C, leaving")
             context.quit(1)
     return rc
 
@@ -295,10 +295,10 @@ def compgen():
     if len(last_word) > 1 and ':' in last_word[1]:
         idx = last_word[1].rfind(':')
         for w in context.complete(line):
-            print w[idx+1:]
+            print(w[idx+1:])
     else:
         for w in context.complete(line):
-            print w
+            print(w)
 
 
 def parse_options():
@@ -338,7 +338,7 @@ def profile_run(context, user_args):
         stats_cmd = "; ".join(['import pstats',
                                's = pstats.Stats("%s")' % options.profile,
                                's.sort_stats("cumulative").print_stats()'])
-        print "python -c '%s' | less" % (stats_cmd)
+        print("python -c '%s' | less" % (stats_cmd))
     return 0
 
 
@@ -360,18 +360,21 @@ def run():
             err_buf.reset_lineno()
             options.batch = True
         user_args = parse_options()
+        if config.core.debug:
+            print(utils.debug_timestamp())
         term.init()
         if options.profile:
             return profile_run(context, user_args)
         else:
             return main_input_loop(context, user_args)
     except KeyboardInterrupt:
-        print "Ctrl-C, leaving"
+        print("Ctrl-C, leaving")
         sys.exit(1)
     except ValueError as e:
         if config.core.debug:
             import traceback
             traceback.print_exc()
+            sys.stdout.flush()
         common_err(str(e))
 
 # vim:ts=4:sw=4:et:
