@@ -43,7 +43,7 @@ _ROLE2_RE = re.compile(r"role=(.+)$", re.IGNORECASE)
 _TARGET_RE = re.compile(r'([^:]+):$')
 _TARGET_ATTR_RE = re.compile(r'attr:([\w-]+)=([\w-]+)$', re.IGNORECASE)
 _TARGET_PATTERN_RE = re.compile(r'pattern:(.+)$', re.IGNORECASE)
-TERMINATORS = ('params', 'meta', 'utilization', 'operations', 'op', 'rule', 'attributes')
+TERMINATORS = ('params', 'meta', 'utilization', 'operations', 'op', 'op_params', 'op_meta', 'rule', 'attributes')
 
 
 class ParseError(Exception):
@@ -134,6 +134,7 @@ validator = Validation()
 
 class BaseParser(object):
     _BINOP_RE = None
+    _VALUE_SOURCE_RE = None
 
     def parse(self, cmd):
         "Called by do_parse(). Raises ParseError if parsing fails."
@@ -182,7 +183,7 @@ class BaseParser(object):
         tok = self.current_token()
         if not tok:
             return None
-        if isinstance(rx, basestring):
+        if isinstance(rx, str):
             if not rx.endswith('$'):
                 rx = rx + '$'
             self._lastmatch = re.match(rx, tok, re.IGNORECASE)
@@ -205,7 +206,7 @@ class BaseParser(object):
         if not self.try_match(rx):
             if errmsg:
                 self.err(errmsg)
-            elif isinstance(rx, basestring):
+            elif isinstance(rx, str):
                 self.err("Expected " + rx)
             else:
                 self.err("Expected " + rx.pattern.rstrip('$'))
@@ -215,10 +216,10 @@ class BaseParser(object):
         """
         After a successful match, returns
         the groups generated by the match.
-
-        FIXME: check if _lastmatch is a list
         """
-        return self._lastmatch.group(idx)
+        if hasattr(self._lastmatch, "group"):
+            return self._lastmatch.group(idx)
+        return None
 
     def lastmatch(self):
         return self._lastmatch
@@ -428,7 +429,7 @@ class BaseParser(object):
         generator which matches attr_lists
         name_map: maps CLI name to XML name
         """
-        to_match = '|'.join(name_map.keys())
+        to_match = '|'.join(list(name_map.keys()))
         if self.try_match(to_match):
             name = self.matched(0).lower()
             yield self.match_attr_list(name, name_map[name], terminator=terminator)
@@ -526,12 +527,17 @@ class BaseParser(object):
             self.match(self._BINOP_RE)
             optype = self.matched(2)
             binop = self.matched(3)
-            val = self.match_any()
-            node = xmlutil.new('expression',
-                               operation=binop,
-                               attribute=attr,
-                               value=val)
+            node = xmlutil.new('expression', operation=binop, attribute=attr)
             xmlutil.maybe_set(node, 'type', optype)
+            val = self.match_any()
+            if not self._VALUE_SOURCE_RE:
+                self._VALUE_SOURCE_RE = re.compile(r"^(?P<val_src>[^\s{}]+)({(?P<val>\S+)})?$")
+            val_src_match = re.match(self._VALUE_SOURCE_RE, val)
+            if val_src_match.group('val') is None:
+                node.set('value', val)
+            else:
+                node.set('value', val_src_match.group('val'))
+                node.set('value-source', val_src_match.group('val_src'))
             return node
 
     def match_date(self):
@@ -605,10 +611,14 @@ class BaseParser(object):
         This is so for example: primitive foo Dummy state=1 is accepted when
         params is the implicit initial.
         """
-        names = olist(name_map.keys())
+        names = olist(list(name_map.keys()))
         oplist = olist([op for op in name_map if op.lower() in ('operations', 'op')])
         for op in oplist:
             del name_map[op]
+        bundle_list = olist([op for op in name_map if op.lower()
+                            in ('docker', 'rkt', 'network', 'port-mapping', 'storage', 'primitive')])
+        for bl in bundle_list:
+            del name_map[bl]
         initial = True
         while self.has_tokens():
             t = self.current_token().lower()
@@ -616,7 +626,11 @@ class BaseParser(object):
                 initial = False
                 if t in oplist:
                     self.match_operations(out, t == 'operations')
+                if t in bundle_list:
+                    self.match_container(out, t)
                 else:
+                    if bundle_list:
+                        terminator = ['network', 'storage', 'primitive']
                     for attr_list in self.match_attr_lists(name_map, terminator=terminator):
                         out.append(attr_list)
             elif initial:
@@ -628,6 +642,35 @@ class BaseParser(object):
             else:
                 break
 
+    def match_container(self, out, _type):
+        container_node = None
+        self.match(_type)
+        all_attrs = self.match_nvpairs(minpairs=0, terminator=['network', 'storage', 'meta', 'primitive'])
+
+        if _type != "primitive":
+            exist_node = out.find(_type)
+            if exist_node is None:
+                container_node = xmlutil.new(_type)
+            else:
+                container_node = exist_node
+
+            child_flag = False
+            for nvp in all_attrs:
+                if nvp.get('name') in ['port-mapping', 'storage-mapping']:
+                    inst_attrs = xmlutil.child(container_node, nvp.get('name'))
+                    child_flag = True
+                    continue
+                if child_flag:
+                    inst_attrs.set(nvp.get('name'), nvp.get('value'))
+                else:
+                    container_node.set(nvp.get('name'), nvp.get('value'))
+            out.append(container_node)
+
+        else:
+            if len(all_attrs) != 1 or all_attrs[0].get('value'):
+                self.err("Expected primitive reference, got {}".format(", ".join("{}={}".format(nvp.get('name'), nvp.get('value') or "") for nvp in all_attrs)))
+            xmlutil.child(out, 'crmsh-ref', id=all_attrs[0].get('name'))
+
     def match_op(self, out, pfx='op'):
         """
         op <optype> [<n>=<v> ...]
@@ -649,11 +692,19 @@ class BaseParser(object):
         inst_attrs = None
         for nvp in all_attrs:
             if nvp.get('name') in valid_attrs:
+                if inst_attrs is not None:
+                    self.err("Attribute order error: {} must appear before any instance attribute".format(nvp.get('name')))
                 node.set(nvp.get('name'), nvp.get('value'))
             else:
                 if inst_attrs is None:
                     inst_attrs = xmlutil.child(node, 'instance_attributes')
                 inst_attrs.append(nvp)
+        if inst_attrs is not None:
+            node.append(inst_attrs)
+        for attr_list in self.match_attr_lists({'op_params': 'instance_attributes',
+                                                'op_meta': 'meta_attributes'},
+                                               implicit_initial='op_params'):
+            node.append(attr_list)
         out.append(node)
 
     def match_operations(self, out, match_id):
@@ -727,7 +778,7 @@ def parse_node(self, cmd):
     return out
 
 
-@parser_for('primitive', 'group', 'clone', 'ms', 'master', 'rsc_template')
+@parser_for('primitive', 'group', 'clone', 'ms', 'master', 'rsc_template', 'bundle')
 class ResourceParser(BaseParser):
     def match_ra_type(self, out):
         "[<class>:[<provider>:]]<type>"
@@ -820,6 +871,19 @@ class ResourceParser(BaseParser):
             xmlutil.child(out, 'crmsh-ref', id=child)
         return out
 
+    def parse_bundle(self):
+        out = xmlutil.new('bundle')
+        out.set('id', self.match_identifier())
+        xmlutil.maybe_set(out, 'description', self.try_match_description())
+        self.match_arguments(out, {'docker': 'docker',
+                                   'rkt': 'rkt',
+                                   'network': 'network',
+                                   'port-mapping': 'port-mapping',
+                                   'storage': 'storage',
+                                   'meta': 'meta_attributes',
+                                   'primitive': 'primitive'})
+        return out
+
 
 @parser_for('location', 'colocation', 'collocation', 'order', 'rsc_ticket')
 class ConstraintParser(BaseParser):
@@ -1039,9 +1103,9 @@ def property_parser(self, cmd):
 
 @parser_for('fencing-topology', 'fencing_topology')
 class FencingOrderParser(BaseParser):
-    """
+    '''
     <fencing-topology>
-    <fencing-level id=<id> target=<text> index=<+int> devices=r"\w,\w..."/>
+    <fencing-level id=<id> target=<text> index=<+int> devices=r"\\w,\\w..."/>
     </fencing-topology>
 
     new:
@@ -1052,7 +1116,7 @@ class FencingOrderParser(BaseParser):
 
     <fencing-topology>
     <fencing-level id=<id> target-attribute=<text> target-value=<text>
-                   index=<+int> devices=r"\w,\w..."/>
+                   index=<+int> devices=r"\\w,\\w..."/>
     </fencing-topology>
 
     from 1.1.14 on, target can be a regexp pattern:
@@ -1061,14 +1125,14 @@ class FencingOrderParser(BaseParser):
 
     <fencing-topology>
     <fencing-level id=<id> target-pattern=<pattern>
-                   index=<+int> devices=r"\w,\w..."/>
+                   index=<+int> devices=r"\\w,\\w..."/>
     </fencing-topology>
 
     fencing-topology \
       pcmk-1: poison-pill power \
       pcmk-2: disk,network power
 
-    """
+    '''
     def parse(self, cmd):
         self.begin(cmd)
         if not self.try_match("fencing-topology"):
@@ -1107,7 +1171,7 @@ class FencingOrderParser(BaseParser):
             lvl_generator = wrap_levels
 
         out = xmlutil.new('fencing-topology')
-        targets = defaultdict(repeat(1).next)
+        targets = defaultdict(repeat(1).__next__)
         for target, devices in lvl_generator():
             if isinstance(target, tuple):
                 if target[0] is None:
@@ -1250,7 +1314,7 @@ class AclParser(BaseParser):
 
     def _add_rule(self):
         rule = xmlutil.new(self.match(_ACL_RIGHT_RE).lower())
-        eligible_specs = constants.acl_spec_map.values()
+        eligible_specs = list(constants.acl_spec_map.values())
         while self.has_tokens():
             a = self._expand_shortcuts(self.current_token().split(':', 1))
             if len(a) != 2 or a[0] not in eligible_specs:
@@ -1410,11 +1474,35 @@ def parse_alert(self, cmd):
     if desc is not None:
         out.attrib['description'] = desc
     rcount = 1
+    root_selector = [None]
+
+    def wrap_select(tag):
+        if tag[0] is None:
+            tag[0] = xmlutil.child(out, 'select')
+        return tag[0]
+
     while self.has_tokens():
         if self.current_token() in ('attributes', 'meta'):
             self.match_arguments(out, {'attributes': 'instance_attributes',
                                        'meta': 'meta_attributes'},
-                                 terminator=['attributes', 'meta', 'to'])
+                                 terminator=['attributes', 'meta', 'to', 'select'])
+            continue
+        if self.current_token() == 'select':
+            selector_types = ('nodes', 'fencing', 'resources', 'attributes')
+            self.match('select')
+            root_selector[0] = None
+            while self.current_token() in selector_types:
+                selector = self.match_identifier()
+                if selector == 'attributes':
+                    if not self.try_match('{'):
+                        self.rewind()
+                        break
+                seltag = xmlutil.child(wrap_select(root_selector), 'select_{}'.format(selector))
+                if selector == 'attributes':
+                    while self.current_token() != '}':
+                        name = self.match_identifier()
+                        xmlutil.child(seltag, 'attribute', name=name)
+                    self.match('}')
             continue
         self.match('to')
         rid = '%s-recipient-%s' % (alertid, rcount)
@@ -1610,10 +1698,11 @@ def parse(s, comments=None):
     if comments is None:
         comments = []
 
-    if isinstance(s, unicode):
+    if isinstance(s, str):
         try:
             s = s.encode('ascii', errors='xmlcharrefreplace')
-        except Exception, e:
+            s = s.decode('utf-8')
+        except Exception as e:
             common_err(e)
             return False
     if isinstance(s, str):
@@ -1623,7 +1712,7 @@ def parse(s, comments=None):
         if s.startswith('xml'):
             try:
                 s = [x for p in lines2cli(s) for x in p.split()]
-            except ValueError, e:
+            except ValueError as e:
                 common_err(e)
                 return False
         else:
