@@ -4,7 +4,6 @@
 
 import optparse
 import re
-from . import options as cmdoptions
 from . import command
 from . import utils
 from .msg import err_buf
@@ -81,6 +80,8 @@ class Cluster(command.UI):
         '''
         try:
             utils.start_service("pacemaker")
+            if utils.is_qdevice_configured():
+                utils.start_service("corosync-qdevice")
             err_buf.info("Cluster services started")
         except IOError as err:
             context.fatal_error(str(err))
@@ -93,6 +94,8 @@ class Cluster(command.UI):
         Stops the cluster services on this node
         '''
         try:
+            if utils.is_qdevice_configured():
+                utils.stop_service("corosync-qdevice")
             utils.stop_service("corosync")
             err_buf.info("Cluster services stopped")
         except IOError as err:
@@ -172,6 +175,7 @@ Stage can be one of:
     vgfs        Create volume group and filesystem (ocfs2 template only,
                 requires -o <dev>)
     admin       Create administration virtual IP (optional)
+    qdevice     Configure qdevice and qnetd
 
 Note:
   - If stage is not specified, the script will run through each stage
@@ -184,19 +188,66 @@ Note:
     will be skipped.
 """, add_help_option=False)
 
-        for _options in cmdoptions.CLUSTER_INIT_BASE_OPTIONS:
-            parser.add_option(*_options[0], **_options[-1])
+        parser.add_option("-h", "--help", action="store_true", dest="help", help="Show this help message")
+        parser.add_option("-q", "--quiet", action="store_true", dest="quiet",
+                          help="Be quiet (don't describe what's happening, just do it)")
+        parser.add_option("-y", "--yes", action="store_true", dest="yes_to_all",
+                          help='Answer "yes" to all prompts (use with caution, this is destructive, especially during the "storage" stage. The /root/.ssh/id_rsa key will be overwritten unless the option "--no-overwrite-sshkey" is used)')
+        parser.add_option("-t", "--template", dest="template",
+                          help='Optionally configure cluster with template "name" (currently only "ocfs2" is valid here)')
+        parser.add_option("-n", "--name", metavar="NAME", dest="name", default="hacluster",
+                          help='Set the name of the configured cluster.')
+        parser.add_option("-N", "--nodes", metavar="NODES", dest="nodes",
+                          help='Additional nodes to add to the created cluster. ' +
+                          'May include the current node, which will always be the initial cluster node.')
+        # parser.add_option("--quick-start", dest="quickstart", action="store_true", help="Perform basic system configuration (NTP, watchdog, /etc/hosts)")
+        parser.add_option("-S", "--enable-sbd", dest="diskless_sbd", action="store_true",
+                          help="Enable SBD even if no SBD device is configured (diskless mode)")
+        parser.add_option("-w", "--watchdog", dest="watchdog", metavar="WATCHDOG",
+                          help="Use the given watchdog device")
+        parser.add_option("--no-overwrite-sshkey", action="store_true", dest="no_overwrite_sshkey",
+                          help='Avoid "/root/.ssh/id_rsa" overwrite if "-y" option is used (False by default)')
 
         network_group = optparse.OptionGroup(parser, "Network configuration", "Options for configuring the network and messaging layer.")
-        for _options in cmdoptions.CLUSTER_INIT_NET_OPTIONS:
-            network_group.add_option(*_options[0], **_options[-1])
+        network_group.add_option("-i", "--interface", dest="nic", metavar="IF",
+                                 help="Bind to IP address on interface IF")
+        network_group.add_option("-u", "--unicast", action="store_true", dest="unicast",
+                                 help="Configure corosync to communicate over unicast (UDP), and not multicast. " +
+                                 "Default is multicast unless an environment where multicast cannot be used is detected.")
+        network_group.add_option("-A", "--admin-ip", dest="admin_ip", metavar="IP",
+                                 help="Configure IP address as an administration virtual IP")
+        network_group.add_option("-M", "--multi-heartbeats", action="store_true", dest="second_hb",
+                                 help="Configure corosync with second heartbeat line")
+        network_group.add_option("-I", "--ipv6", action="store_true", dest="ipv6",
+                                 help="Configure corosync use IPv6")
+        network_group.add_option("--qnetd-hostname",
+                                 dest="qdevice", metavar="HOST",
+                                 help="HOST or IP of the QNetd server to be used")
+        network_group.add_option("--qdevice-port",
+                                 dest="qdevice_port", metavar="PORT", type="int", default=5403,
+                                 help="TCP PORT of QNetd server(default:5403)")
+        network_group.add_option("--qdevice-algo",
+                                 dest="qdevice_algo", metavar="ALGORITHM", default="ffsplit",
+                                 help="QNetd decision ALGORITHM(ffsplit/lms, default:ffsplit)")
+        network_group.add_option("--qdevice-tie-breaker",
+                                 dest="qdevice_tie_breaker", metavar="TIE_BREAKER", default="lowest",
+                                 help="QNetd TIE_BREAKER(lowest/highest/valid_node_id, default:lowest)")
+        network_group.add_option("--qdevice-tls",
+                                 dest="qdevice_tls", metavar="TLS", default="on",
+                                 help="Whether using TLS on QDevice/QNetd(on/off/required, default:on)")
+        network_group.add_option("--qdevice-heuristics",
+                                 dest="qdevice_heuristics", metavar="COMMAND",
+                                 help="COMMAND to run with absolute path. For multiple commands, use \";\" to separate(details about heuristics can see man 8 corosync-qdevice)")
         parser.add_option_group(network_group)
 
         storage_group = optparse.OptionGroup(parser, "Storage configuration", "Options for configuring shared storage.")
-        for _options in cmdoptions.CLUSTER_INIT_STORAGE_OPTIONS:
-            storage_group.add_option(*_options[0], **_options[-1])
+        storage_group.add_option("-p", "--partition-device", dest="shared_device", metavar="DEVICE",
+                                 help='Partition this shared storage device (only used in "storage" stage)')
+        storage_group.add_option("-s", "--sbd-device", dest="sbd_device", metavar="DEVICE", action="append",
+                                 help="Block device to use for SBD fencing, use \";\" as separator for multi path")
+        storage_group.add_option("-o", "--ocfs2-device", dest="ocfs2_device", metavar="DEVICE",
+                                 help='Block device to use for OCFS2 (only used in "vgfs" stage)')
         parser.add_option_group(storage_group)
-
         try:
             options, args = parser.parse_args(list(args))
         except:
@@ -226,7 +277,9 @@ Note:
                 options.qdevice,
                 port=options.qdevice_port,
                 algo=options.qdevice_algo,
-                tie_breaker=options.qdevice_tie_breaker)
+                tie_breaker=options.qdevice_tie_breaker,
+                tls=options.qdevice_tls,
+                cmds=options.qdevice_heuristics)
 
         bootstrap.bootstrap_init(
             cluster_name=options.name,
@@ -240,6 +293,7 @@ Note:
             template=options.template,
             admin_ip=options.admin_ip,
             yes_to_all=options.yes_to_all,
+            no_overwrite_sshkey=options.no_overwrite_sshkey,
             unicast=options.unicast,
             second_hb=options.second_hb,
             ipv6=options.ipv6,
@@ -357,6 +411,7 @@ If stage is not specified, each stage will be invoked in sequence.
         parser.add_option("-y", "--yes", help='Answer "yes" to all prompts (use with caution)', action="store_true", dest="yes_to_all")
         parser.add_option("-c", "--cluster-node", dest="cluster_node", help="IP address or hostname of cluster node which will be deleted", metavar="HOST")
         parser.add_option("-F", "--force", dest="force", help="Remove current node", action="store_true")
+        parser.add_option("--qdevice", dest="qdevice", help="Remove QDevice configuration and service from cluster", action="store_true")
         try:
             options, args = parser.parse_args(list(args))
         except:
@@ -371,7 +426,8 @@ If stage is not specified, each stage will be invoked in sequence.
                 cluster_node=None,
                 ui_context=context,
                 quiet=options.quiet,
-                yes_to_all=options.yes_to_all)
+                yes_to_all=options.yes_to_all,
+                qdevice=options.qdevice)
         else:
             for node in args:
                 bootstrap.bootstrap_remove(
@@ -593,7 +649,7 @@ Cluster Description
             context.fatal_error("Timed out waiting for cluster (rc = %s)" % (ret))
 
     @command.skill_level('expert')
-    def do_run(self, context, cmd, node=None):
+    def do_run(self, context, cmd, *nodes):
         '''
         Execute the given command on all nodes/specific node, report outcome
         '''
@@ -606,10 +662,16 @@ Cluster Description
         if not _has_parallax:
             context.fatal_error("python package parallax is needed for this command")
 
-        hosts = utils.list_cluster_nodes()
-        if node and node in hosts:
-            hosts = [node]
+        if nodes:
+            hosts = list(nodes)
+        else:
+            hosts = utils.list_cluster_nodes()
+            if hosts is None:
+                context.fatal_error("failed to get node list from cluster")
+
         opts = parallax.Options()
+        opts.ssh_options = ['StrictHostKeyChecking=no']
+        opts.askpass = utils.check_ssh_passwd_need(hosts)
         for host, result in parallax.call(hosts, cmd, opts).items():
             if isinstance(result, parallax.Error):
                 err_buf.error("[%s]: %s" % (host, result))
