@@ -9,6 +9,7 @@ from . import options
 from .msg import common_err, common_info, common_warn
 from . import ui_utils
 from . import userdir
+from . import constants
 
 
 # import logging
@@ -46,10 +47,20 @@ class Context(object):
         '''
         Execute the given command line.
         '''
+        def trans_to_help(line):
+            line_list = line.split()
+            if line_list[-1] in ["-h", "--help"] and \
+               line_list[-2] == "property":
+                return " ".join(line_list[:-2] + ["help", "property"])
+            else:
+                return line
+
         line = line.strip()
         if not line or line.startswith('#'):
             return True
 
+        line = trans_to_help(line)
+
         self._mark = len(self.stack)
         self._in_transit = False
         self._wait_for_dc = False
@@ -73,16 +84,18 @@ class Context(object):
                     break
             if cmd:
                 rv = self.execute_command() is not False
-        except ValueError, msg:
-            if config.core.debug:
+        except ValueError as msg:
+            if config.core.debug or options.regression_tests:
                 import traceback
                 traceback.print_exc()
+                sys.stdout.flush()
             common_err("%s: %s" % (self.get_qualified_name(), msg))
             rv = False
-        except IOError, msg:
-            if config.core.debug:
+        except IOError as msg:
+            if config.core.debug or options.regression_tests:
                 import traceback
                 traceback.print_exc()
+                sys.stdout.flush()
             common_err("%s: %s" % (self.get_qualified_name(), msg))
             rv = False
         if cmd or (rv is False):
@@ -132,6 +145,12 @@ class Context(object):
                         ret = self.command_info.complete(self, tokens)
                         if tokens:
                             ret = [t for t in ret if t.startswith(tokens[-1])]
+
+                        if not ret or self.command_info.aliases:
+                            if not token in self.current_level().get_completions():
+                                return self.current_level().get_completions()
+                        if self.command_name in self.command_info.aliases and not self.command_args:
+                            return [self.command_name]
                         return ret
                 # reached the end on a valid level.
                 # return the completions for the previous level.
@@ -170,6 +189,15 @@ class Context(object):
         except IOError:
             pass
 
+    def disable_completion(self):
+        import readline
+        readline.parse_and_bind('tab: complete')
+        readline.set_completer(self.disable_completer)
+
+    def disable_completer(self, text, state):
+        # complete nothing
+        return
+
     def clear_readline_cache(self):
         self._rl_line = None
         self._rl_words = []
@@ -187,9 +215,10 @@ class Context(object):
                 self._rl_line = line
                 completions = self.complete(line)
                 if text:
-                    self._rl_words = [w for w in completions if matching(w)]
+                    self._rl_words = [w for w in completions if matching(w) and not w.startswith("_")]
                 else:
-                    self._rl_words = list(completions)
+                    self._rl_words = [w for w in completions if not w.startswith("_")]
+
             except Exception:  # , msg:
                 # logging.exception(msg)
                 self.clear_readline_cache()
@@ -200,6 +229,8 @@ class Context(object):
             ret = None
         # logging.debug("line:%s, text:%s, ret:%s, state:%s", repr(line), repr(text), ret, state)
         if not text or (ret and line.split()[-1].endswith(ret)):
+            if ret == "id=":
+                return ret
             return ret + ' '
         return ret
 
@@ -318,7 +349,10 @@ class Context(object):
         '''
         ok = self.current_level().end_game()
         if options.interactive and not options.batch:
-            print "bye"
+            if constants.need_reset:
+                utils.ext_cmd("reset")
+            else:
+                print("bye")
         if ok is False and rc == 0:
             rc = 1
         sys.exit(rc)
