@@ -7,20 +7,37 @@ from . import completers
 from . import utils
 from .msg import err_buf
 from . import corosync
+from . import parallax
 
 
 def _push_completer(args):
     try:
         n = utils.list_cluster_nodes()
         n.remove(utils.this_node())
+        if args[-1] in n:
+            # continue complete
+            return [args[-1]]
+        for item in args:
+            if item in n:
+                n.remove(item)
         return n
     except:
         n = []
 
 
-def _all_nodes(args):
+def _diff_nodes(args):
     try:
-        return utils.list_cluster_nodes()
+        if len(args) > 3:
+            return []
+        n = utils.list_cluster_nodes()
+        if args[-1] in n:
+            # continue complete
+            return [args[-1]]
+        for item in args:
+            if item in n:
+                # remove already complete item
+                n.remove(item)
+        return n
     except:
         return []
 
@@ -40,12 +57,45 @@ class Corosync(command.UI):
             return False
         return corosync.check_tools()
 
-    def do_status(self, context):
+    @command.completers(completers.choice(['ring', 'quorum', 'qnetd']))
+    def do_status(self, context, status_type="ring"):
         '''
-        Quick cluster health status. Corosync status...
+        Quick cluster health status. Corosync status or QNetd status
         '''
-        print corosync.cfgtool('-s')[1]
-        print corosync.quorumtool('-s')[1]
+        rc, _ = utils.get_stdout('systemctl -q is-active corosync.service')
+        if rc != 0:
+            err_buf.error("corosync.service is not running!")
+            return False
+
+        if status_type == "ring":
+            print(corosync.cfgtool('-s')[1])
+            return
+        if status_type == "quorum":
+            print(corosync.quorumtool('-s')[1])
+            return
+        if status_type == "qnetd":
+            if not utils.is_qdevice_configured():
+                err_buf.error("QDevice/QNetd not configured!")
+                return False
+            cluster_name = corosync.get_value('totem.cluster_name')
+            if not cluster_name:
+                err_buf.error("cluster_name not configured!")
+                return False
+            qnetd_addr = corosync.get_value('quorum.device.net.host')
+            if not qnetd_addr:
+                err_buf.error("host for qnetd not configured!")
+                return False
+
+            cmd = "corosync-qnetd-tool -lv -c {}".format(cluster_name)
+            try:
+                result = parallax.parallax_call([qnetd_addr], cmd)
+            except ValueError as err:
+                err_buf.error(err)
+                return False
+            _, qnetd_result_stdout, _ = result[0][1]
+            if qnetd_result_stdout:
+                print(utils.to_ascii(qnetd_result_stdout))
+            return
 
     @command.skill_level('administrator')
     def do_reload(self, context):
@@ -75,7 +125,7 @@ class Corosync(command.UI):
         '''
         return corosync.pull_configuration(node)
 
-    @command.completers_repeating(_all_nodes)
+    @command.completers_repeating(_diff_nodes)
     def do_diff(self, context, *nodes):
         '''
         Compare corosync configuration between nodes.
@@ -96,7 +146,7 @@ class Corosync(command.UI):
         cfg = corosync.conf()
         try:
             utils.edit_file_ext(cfg, template='')
-        except IOError, e:
+        except IOError as e:
             context.fatal_error(str(e))
 
     def do_show(self, context):
@@ -127,8 +177,12 @@ class Corosync(command.UI):
     @command.name('del-node')
     @command.alias('del_node')
     @command.skill_level('administrator')
+    @command.completers(_push_completer)
     def do_delnode(self, context, name):
         "Remove a node from the corosync nodelist"
+        transport = corosync.get_value('totem.transport')
+        if not transport or transport != "udpu":
+            context.fatal_error("Only support udpu transport now")
         corosync.del_node(name)
 
     @command.skill_level('administrator')
@@ -136,7 +190,7 @@ class Corosync(command.UI):
     def do_get(self, context, path):
         "Get a corosync configuration value"
         for v in corosync.get_values(path):
-            print v
+            print(v)
 
     @command.skill_level('administrator')
     def do_set(self, context, path, value):
