@@ -1,5 +1,5 @@
 # Copyright (C) 2008-2011 Dejan Muhamedagic <dmuhamedagic@suse.de>
-# Copyright (C) 2013 Kristoffer Gronlund <kgronlund@suse.com>
+# Copyright (C) 2013-2018 Kristoffer Gronlund <kgronlund@suse.com>
 # See COPYING for license information.
 
 from . import command
@@ -30,23 +30,26 @@ def rm_meta_attribute(node, attr, l, force_children=False):
                 (xmlutil.is_child_rsc(c) and not c.getparent().tag == "group"):
             rm_meta_attribute(c, attr, l, force_children=force_children)
 
-
-def get_children_with_different_attr(node, attr, value):
+def get_children_with_attr(node, attr):
     l = []
     for p in node.xpath(".//primitive"):
         diff_attr = False
         for meta_set in xmlutil.get_set_nodes(p, "meta_attributes", create=False):
             p_value = xmlutil.get_attr_value(meta_set, attr)
-            if p_value is not None and p_value != value:
+            if p_value is not None:
                 diff_attr = True
                 break
         if diff_attr:
             l.append(p)
     return l
 
-
 def set_deep_meta_attr_node(target_node, attr, value):
     nvpair_l = []
+    conflicting_attr = ''
+    if 'maintenance' == attr:
+        conflicting_attr = 'is-managed'
+    if 'is-managed' == attr:
+        conflicting_attr = 'maintenance'
     if xmlutil.is_clone(target_node):
         for c in target_node.iterchildren():
             if xmlutil.is_child_rsc(c):
@@ -54,16 +57,35 @@ def set_deep_meta_attr_node(target_node, attr, value):
     if config.core.manage_children != "never" and \
             (xmlutil.is_group(target_node) or
              (xmlutil.is_clone(target_node) and xmlutil.cloned_el(target_node) == "group")):
-        odd_children = get_children_with_different_attr(target_node, attr, value)
+        # maintenance and is-managed attributes conflict,
+        # ask the user if he wants to leave both.
+        if '' != conflicting_attr:
+            odd_children = get_children_with_attr(target_node, conflicting_attr)
+            for c in odd_children:
+                if config.core.manage_children == "always" or \
+                        (config.core.manage_children == "ask" and
+                        utils.ask("'%s' conflicts with '%s' attribute. Remove '%s' for child resource %s?" %
+                                (attr, conflicting_attr, conflicting_attr, c.get("id")))):
+                    common_debug("force remove meta attr %s from %s" %
+                                (conflicting_attr, c.get("id")))
+                    rm_meta_attribute(c, conflicting_attr, nvpair_l, force_children=True)
+        # remove the same attributes from all children (if wanted)
+        odd_children = get_children_with_attr(target_node, attr)
         for c in odd_children:
             if config.core.manage_children == "always" or \
                     (config.core.manage_children == "ask" and
-                     utils.ask("Do you want to override %s for child resource %s?" %
+                     utils.ask("Do you want to override '%s' for child resource %s?" %
                                (attr, c.get("id")))):
                 common_debug("force remove meta attr %s from %s" %
                              (attr, c.get("id")))
                 rm_meta_attribute(c, attr, nvpair_l, force_children=True)
     xmlutil.rmnodes(list(set(nvpair_l)))
+    if '' != conflicting_attr:
+        nvpairs = target_node.xpath("./meta_attributes/nvpair[@name='%s']" % (conflicting_attr))
+        if len(nvpairs) > 0:
+            if (utils.ask("'%s' conflicts with '%s' attribute. Remove '%s' for resource %s?" %
+                        (attr, conflicting_attr, conflicting_attr, target_node.get("id")))):
+                xmlutil.rmnodes(list(set(nvpairs)))
     xmlutil.xml_processnodes(target_node,
                              xmlutil.is_emptynvpairs, xmlutil.rmnodes)
 
@@ -129,7 +151,7 @@ def set_deep_meta_attr(rsc, attr, value, commit=True):
         return False
     while any(obj for obj in objs if obj.obj_type == 'tag'):
         objs = list(flatten(resolve(obj) for obj in objs))
-    objs = filter(is_resource, objs)
+    objs = list(filter(is_resource, objs))
     common_debug("set_deep_meta_attr: %s" % (', '.join([obj.obj_id for obj in objs])))
     if not objs:
         common_error("Resource not found: %s" % (rsc))
@@ -150,17 +172,6 @@ def set_deep_meta_attr(rsc, attr, value, commit=True):
     return True
 
 
-def cleanup_resource(rsc, node='', force=False):
-    if not utils.is_name_sane(rsc) or not utils.is_name_sane(node):
-        return False
-    forces = " -f" if force else ""
-    if not node:
-        rc = utils.ext_cmd((RscMgmt.rsc_cleanup_all % (rsc)) + forces) == 0
-    else:
-        rc = utils.ext_cmd((RscMgmt.rsc_cleanup % (rsc, node)) + forces) == 0
-    return rc
-
-
 _attrcmds = compl.choice(['delete', 'set', 'show'])
 _raoperations = compl.choice(constants.ra_operations)
 
@@ -171,39 +182,38 @@ class RscMgmt(command.UI):
     '''
     name = "resource"
 
-    rsc_status_all = "crm_resource -L"
-    rsc_status = "crm_resource --locate -r '%s'"
-    rsc_showxml = "crm_resource -q -r '%s'"
-    rsc_setrole = "crm_resource --meta -r '%s' -p target-role -v '%s'"
-    rsc_migrate = "crm_resource --quiet --move -r '%s' %s"
-    rsc_unmigrate = "crm_resource --quiet --clear -r '%s'"
-    rsc_ban = "crm_resource --ban -r '%s' %s"
-    rsc_cleanup = "crm_resource -C -r '%s' -H '%s'"
-    rsc_cleanup_all = "crm_resource -C -r '%s'"
-    rsc_maintenance = "crm_resource -r '%s' --meta -p maintenance -v '%s'"
+    rsc_status_all = "crm_resource --list"
+    rsc_status = "crm_resource --locate --resource '%s'"
+    rsc_showxml = "crm_resource --query-xml --resource '%s'"
+    rsc_setrole = "crm_resource --meta --resource '%s' --set-parameter target-role --parameter-value '%s'"
+    rsc_migrate = "crm_resource --quiet --move --resource '%s' %s"
+    rsc_unmigrate = "crm_resource --quiet --clear --resource '%s'"
+    rsc_ban = "crm_resource --ban --resource '%s' %s"
+    rsc_maintenance = "crm_resource --resource '%s' --meta --set-parameter maintenance --parameter-value '%s'"
     rsc_param = {
-        'set': "crm_resource -r '%s' -p '%s' -v '%s'",
-        'delete': "crm_resource -r '%s' -d '%s'",
-        'show': "crm_resource -r '%s' -g '%s'",
-        'get': "crm_resource -r '%s' -g '%s'",
+        'set': "crm_resource --resource '%s' --set-parameter '%s' --parameter-value '%s'",
+        'delete': "crm_resource --resource '%s' --delete-parameter '%s'",
+        'show': "crm_resource --resource '%s' --get-parameter '%s'",
+        'get': "crm_resource --resource '%s' --get-parameter '%s'",
     }
     rsc_meta = {
-        'set': "crm_resource --meta -r '%s' -p '%s' -v '%s'",
-        'delete': "crm_resource --meta -r '%s' -d '%s'",
-        'show': "crm_resource --meta -r '%s' -g '%s'",
-        'get': "crm_resource --meta -r '%s' -g '%s'",
+        'set': "crm_resource --meta --resource '%s' --set-parameter '%s' --parameter-value '%s'",
+        'delete': "crm_resource --meta --resource '%s' --delete-parameter '%s'",
+        'show': "crm_resource --meta --resource '%s' --get-parameter '%s'",
+        'get': "crm_resource --meta --resource '%s' --get-parameter '%s'",
     }
     rsc_failcount = {
-        'set': "crm_attribute -t status -n 'fail-count-%s' -N '%s' -v '%s' -d 0",
+        'set': "crm_attribute -t status -n 'fail-count-%s' -N '%s' -v '%s'",
+        'set_p': "crm_attribute -t status -P 'fail-count-%s' -N '%s' -v '%s'",
         'delete': "crm_failcount -D -r %s -N %s",
         'show': "crm_failcount -G -r %s -N %s",
         'get': "crm_failcount -G -r %s -N %s",
     }
     rsc_utilization = {
-        'set': "crm_resource -z -r '%s' -p '%s' -v '%s'",
-        'delete': "crm_resource -z -r '%s' -d '%s'",
-        'show': "crm_resource -z -r '%s' -g '%s'",
-        'get': "crm_resource -z -r '%s' -g '%s'",
+        'set': "crm_resource --utilization --resource '%s' --set-parameter '%s' --parameter-value '%s'",
+        'delete': "crm_resource --utilization --resource '%s' --delete-parameter '%s'",
+        'show': "crm_resource --utilization --resource '%s' --get-parameter '%s'",
+        'get': "crm_resource --utilization --resource '%s' --get-parameter '%s'",
     }
     rsc_secret = {
         'set': "cibsecret set '%s' '%s' '%s'",
@@ -214,10 +224,29 @@ class RscMgmt(command.UI):
         'get': "cibsecret get '%s' '%s'",
         'check': "cibsecret check '%s' '%s'",
     }
-    rsc_refresh = "crm_resource -C"
-    rsc_refresh_node = "crm_resource -C -H '%s'"
-    rsc_reprobe = "crm_resource -C"
-    rsc_reprobe_node = "crm_resource -C -H '%s'"
+
+    def _refresh_cleanup(self, action, rsc, node, force):
+        """
+        Implements the refresh and cleanup commands.
+        """
+        if rsc == "force":
+            rsc, force = None, True
+        if node == "force":
+            node, force = None, True
+        cmd = ["crm_resource", "--" + action]
+        if rsc:
+            if not utils.is_name_sane(rsc):
+                return False
+            cmd.append("--resource")
+            cmd.append(rsc)
+        if node:
+            if not utils.is_name_sane(node):
+                return False
+            cmd.append("--node")
+            cmd.append(node)
+        if force:
+            cmd.append("--force")
+        return utils.ext_cmd(" ".join(cmd)) == 0
 
     def requires(self):
         for program in ('crm_resource', 'crm_attribute'):
@@ -273,7 +302,7 @@ class RscMgmt(command.UI):
         return rc
 
     @command.wait
-    @command.completers(compl.resources)
+    @command.completers(compl.resources_stopped)
     def do_start(self, context, *resources):
         "usage: start <rsc> [<rsc> ...]"
         if len(resources) == 0:
@@ -281,7 +310,7 @@ class RscMgmt(command.UI):
         return self._commit_meta_attrs(context, resources, "target-role", "Started")
 
     @command.wait
-    @command.completers(compl.resources)
+    @command.completers(compl.resources_started)
     def do_stop(self, context, *resources):
         "usage: stop <rsc> [<rsc> ...]"
         if len(resources) == 0:
@@ -373,9 +402,9 @@ class RscMgmt(command.UI):
 
         opts = ''
         if node:
-            opts = "--node='%s'" % node
+            opts = "--node '%s'" % node
         if lifetime:
-            opts = "%s --lifetime='%s'" % (opts, lifetime)
+            opts = "%s --lifetime '%s'" % (opts, lifetime)
         if force or config.core.force:
             opts = "%s --force" % opts
         rc = utils.ext_cmd(self.rsc_migrate % (rsc, opts))
@@ -403,9 +432,9 @@ class RscMgmt(command.UI):
                 context.fatal_error("Not our node: " + node)
         opts = ''
         if node:
-            opts = "--node='%s'" % node
+            opts = "--node '%s'" % node
         if lifetime:
-            opts = "%s --lifetime='%s'" % (opts, lifetime)
+            opts = "%s --lifetime '%s'" % (opts, lifetime)
         if force:
             opts = "%s --force" % opts
         rc = utils.ext_cmd(self.rsc_ban % (rsc, opts))
@@ -432,14 +461,9 @@ class RscMgmt(command.UI):
     @command.skill_level('administrator')
     @command.wait
     @command.completers(compl.resources, compl.nodes)
-    def do_cleanup(self, context, resource, *args):
-        "usage: cleanup <rsc> [<node>] [force]"
-        # Cleanup a resource on a node. Omit node to cleanup on
-        # all live nodes.
-        argl = list(args)
-        force = "force" in utils.fetch_opts(argl, ["force"]) or config.core.force
-        node = argl[0] if len(argl) > 0 else ''
-        return cleanup_resource(resource, node, force=force)
+    def do_cleanup(self, context, rsc=None, node=None, force=False):
+        "usage: cleanup [<rsc>] [<node>] [force]"
+        return self._refresh_cleanup("cleanup", rsc, node, force)
 
     @command.wait
     @command.completers(compl.resources, compl.nodes)
@@ -460,11 +484,75 @@ class RscMgmt(command.UI):
 
     @command.wait
     @command.completers(compl.resources, _attrcmds, compl.nodes)
-    def do_failcount(self, context, rsc, cmd, node, value=None):
+    def do_failcount(self, context, rsc, cmd, node, value=None, operation=None, interval=None):
         """usage:
-        failcount <rsc> set <node> <value>
+        failcount <rsc> set <node> <value> [operation] [interval]
         failcount <rsc> delete <node>
         failcount <rsc> show <node>"""
+        def sec_to_ms(s):
+            return s + '000'
+
+        def ms_to_sec(m):
+            return m[:len(m)-3]
+
+        if rsc not in compl.resources():
+            context.fatal_error("Resource {} not exists in this cluster".format(rsc))
+        valid_cmd_list = ["set", "delete", "show"]
+        if cmd not in valid_cmd_list:
+            context.fatal_error("{} is not valid command(should be one of {})".format(cmd, valid_cmd_list))
+        nodeid = utils.get_nodeid_from_name(node)
+        if nodeid is None:
+            context.fatal_error("Node {} not in this cluster".format(node))
+
+        if cmd == "set":
+            # query the current failcount status
+            query_cmd = "cibadmin -Ql --xpath '/cib/status/node_state[@id='{}']'".format(nodeid)
+            rc, out, err = utils.get_stdout_stderr(query_cmd)
+            if rc != 0:
+                context.fatal_error(err)
+
+            # try to get failcount dict {operation:interval}
+            import re
+            failcount_res = re.findall(r'fail-count-{}#(.*)_([0-9]+)'.format(rsc), out)
+            if not failcount_res:
+                context.fatal_error("No failcount on node {} for resource {}".format(node, rsc))
+            failcount_dict = dict(failcount_res)
+
+            # validate for operation and interval
+            if operation and operation not in failcount_dict.keys():
+                context.fatal_error("Usage: failcount <rsc> set <node> <value> [operation] [interval]\n\
+                           Should specify operation between \"{}\"".format(' '.join(failcount_dict.keys())))
+            if (operation and interval) and (operation, sec_to_ms(interval)) not in failcount_res:
+                context.fatal_error("Usage: failcount <rsc> set <node> <value> [operation] [interval]\n\
+                           Should specify (operation, interval) between {}".\
+                           format([(op, ms_to_sec(inter)) for op, inter in failcount_res]))
+
+            # just one failcount entry
+            if len(failcount_res) == 1:
+                operation = failcount_res[0][0]
+                interval = failcount_dict[operation]
+                rsc = '{}#{}_{}'.format(rsc, operation, interval)
+
+            # multiple failcount entries for this resource and node
+            if len(failcount_res) > 1:
+                if operation and interval:
+                    rsc = '{}#{}_{}'.format(rsc, operation, sec_to_ms(interval))
+                elif int(value) == 0:
+                    # using '-P' option of 'crm_attribute' command
+                    cmd = "set_p"
+                    if operation:
+                        op_interval_str = '|'.join(['{}_{}'.format(operation, inter) for op, inter in failcount_res if op==operation])
+                    else:
+                        op_interval_str = '|'.join(['{}_{}'.format(op, inter) for op, inter in failcount_res])
+                    rsc = '{}#({})'.format(rsc, op_interval_str)
+                else:
+                    # value != 0
+                    if operation and len([op for op, _ in failcount_res if op == operation]) == 1:
+                        rsc = '{}#{}_{}'.format(rsc, operation, failcount_dict[operation])
+                    else:
+                        context.fatal_error("Should specify (operation, interval) between {}".
+                                format([(op, ms_to_sec(inter)) for op, inter in failcount_res]))
+
         return ui_utils.manage_attr(context.get_command_name(), self.rsc_failcount,
                                     rsc, cmd, node, value)
 
@@ -515,26 +603,11 @@ class RscMgmt(command.UI):
         return ui_utils.manage_attr(context.get_command_name(), self.rsc_utilization,
                                     rsc, cmd, attr, value)
 
+    @command.alias('reprobe')
     @command.completers(compl.nodes)
-    def do_refresh(self, context, *args):
-        'usage: refresh [<node>]'
-        if len(args) == 1:
-            if not utils.is_name_sane(args[0]):
-                return False
-            return utils.ext_cmd(self.rsc_refresh_node % args[0]) == 0
-        else:
-            return utils.ext_cmd(self.rsc_refresh) == 0
-
-    @command.wait
-    @command.completers(compl.nodes)
-    def do_reprobe(self, context, *args):
-        'usage: reprobe [<node>]'
-        if len(args) == 1:
-            if not utils.is_name_sane(args[0]):
-                return False
-            return utils.ext_cmd(self.rsc_reprobe_node % args[0]) == 0
-        else:
-            return utils.ext_cmd(self.rsc_reprobe) == 0
+    def do_refresh(self, context, rsc=None, node=None, force=False):
+        'usage: refresh [<rsc>] [<node>] [force]'
+        return self._refresh_cleanup("refresh", rsc, node, force)
 
     @command.wait
     @command.completers(compl.resources, compl.choice(['on', 'off', 'true', 'false']))
@@ -547,7 +620,7 @@ class RscMgmt(command.UI):
             on_off = 'true'
         else:
             on_off = 'false'
-        return utils.ext_cmd(self.rsc_maintenance % (resource, on_off)) == 0
+        return self._commit_meta_attr(context, resource, "maintenance", on_off)
 
     def _get_trace_rsc(self, rsc_id):
         if not cib_factory.refresh():
@@ -638,7 +711,7 @@ class RscMgmt(command.UI):
 
     def _remove_trace(self, rsc, op_node):
         from lxml import etree
-        common_debug("op_node: %s" % (etree.tostring(op_node)))
+        common_debug("op_node: %s" % (xmlutil.xml_tostring(op_node)))
         op_node = rsc.del_op_attr(op_node, constants.trace_ra_attr)
         if rsc.is_dummy_operation(op_node):
             rsc.del_operation(op_node)
