@@ -15,7 +15,7 @@ import bz2
 import fnmatch
 import gc
 import ipaddress
-from contextlib import contextmanager
+from contextlib import contextmanager, closing
 from . import config
 from . import userdir
 from . import constants
@@ -1760,13 +1760,23 @@ def list_cluster_nodes():
         return None
 
     try:
+        # when pacemaker running
         rc, outp = stdout2list(['crm_node', '-l'], stderr_on=False, shell=False)
         if rc == 0:
             return [x for x in [getname(line.split()) for line in outp] if x and x != '(null)']
 
+        # when corosync running
+        ip_list = get_member_iplist()
+        if ip_list:
+            return ip_list
+
+        # static situation
+        cib_path = os.getenv('CIB_file', '/var/lib/pacemaker/cib/cib.xml')
+        if not os.path.isfile(cib_path):
+            return None
         from . import xmlutil
         node_list = []
-        cib = xmlutil.cibdump2elem()
+        cib = xmlutil.file2cib_elem(cib_path)
         if cib is None:
             return None
         for node in cib.xpath('/cib/configuration/nodes/node'):
@@ -1904,18 +1914,18 @@ def sysconfig_set(sysconfig_file, **values):
                 outp += line
             else:
                 matched = False
-            try:
-                key, _ = line.split("=", 1)
-                for k, v in values.items():
-                    if k == key:
-                        matched = True
-                        outp += '%s=%s\n' % (k, doublequote(v))
-                        del values[k]
-                        break
-                if not matched:
+                try:
+                    key, _ = line.split("=", 1)
+                    for k, v in values.items():
+                        if k == key:
+                            matched = True
+                            outp += '%s=%s\n' % (k, doublequote(v))
+                            del values[k]
+                            break
+                    if not matched:
+                        outp += line
+                except ValueError:
                     outp += line
-            except ValueError:
-                outp += line
 
     for k, v in values.items():
         outp += '%s=%s\n' % (k, doublequote(v))
@@ -2166,4 +2176,92 @@ def debug_timestamp():
     return datetime.datetime.now().strftime('%Y/%m/%d %H:%M:%S')
 
 
+def get_member_iplist():
+    rc, out, err= get_stdout_stderr("corosync-cmapctl -b runtime.totem.pg.mrp.srp.members")
+    if rc != 0:
+        common_debug(err)
+        return None
+
+    ip_list = []
+    for line in out.split('\n'):
+        match = re.search(r'ip\((.*?)\)', line)
+        if match:
+            ip_list.append(match.group(1))
+    return ip_list
+
+
+def check_ssh_passwd_need(hosts):
+    ssh_options = "-o StrictHostKeyChecking=no -o EscapeChar=none -o ConnectTimeout=15"
+    for host in hosts:
+        ssh_cmd = "ssh {} -T -o Batchmode=yes {} true".format(ssh_options, host)
+        rc, _, _ = get_stdout_stderr(ssh_cmd)
+        if rc != 0:
+            return True
+    return False
+
+
+def check_port_open(host, port):
+    import socket
+
+    with closing(socket.socket(socket.AF_INET, socket.SOCK_STREAM)) as sock:
+        if sock.connect_ex((host, port)) == 0:
+            return True
+        else:
+            return False
+
+
+def valid_port(port):
+    return int(port) >= 1024 and int(port) <= 65535
+
+
+def is_qdevice_configured():
+    from . import corosync
+    return corosync.get_value("quorum.device.model") == "net"
+
+
+def is_qdevice_tls_on():
+    from . import corosync
+    return corosync.get_value("quorum.device.net.tls") == "on"
+
+
+def get_nodeinfo_from_cmaptool():
+    nodeid_ip_dict = {}
+    rc, out = get_stdout("corosync-cmapctl -b runtime.totem.pg.mrp.srp.members")
+    if rc != 0:
+        return nodeid_ip_dict
+
+    for line in out.split('\n'):
+        match = re.search(r'members\.(.*)\.ip', line)
+        if match:
+            node_id = match.group(1)
+            iplist = re.findall(r'[0-9]{1,3}\.[0-9]{1,3}\.[0-9]{1,3}\.[0-9]{1,3}', line)
+            nodeid_ip_dict[node_id] = iplist
+    return nodeid_ip_dict
+
+
+def valid_nodeid(nodeid):
+    from . import bootstrap
+    if not bootstrap.service_is_active('corosync.service'):
+        return False
+
+    for _id, _ in get_nodeinfo_from_cmaptool().items():
+        if _id == nodeid:
+            return True
+    return False
+
+
+def is_unicast():
+    from . import corosync
+    return corosync.get_value("totem.transport") == "udpu"
+
+
+def get_nodeid_from_name(name):
+    rc, out = get_stdout('crm_node -l')
+    if rc != 0:
+        return None
+    res = re.search(r'^([0-9]+) {} '.format(name), out, re.M)
+    if res:
+        return res.group(1)
+    else:
+        return None
 # vim:ts=4:sw=4:et:
